"""Controller holding and managing Afero IoT resources of type `security-system`."""

from aioafero.device import AferoDevice
from aioafero.errors import DeviceNotFound
from aioafero.v1.models import SecuritySystemSensor, SecuritySystemSensorPut, features
from aioafero.v1.models.resource import DeviceInformation, ResourceTypes

from .base import AferoBinarySensor, AferoSensor, AferoState, BaseResourcesController
from .security_system import SENSOR_SPLIT_IDENTIFIER


class SecuritySystemSensorController(BaseResourcesController[SecuritySystemSensor]):
    """Controller holding and managing Afero IoT `security-system` for the sensors."""

    ITEM_TYPE_ID = ResourceTypes.DEVICE
    ITEM_TYPES = [ResourceTypes.SECURITY_SYSTEM_SENSOR]
    ITEM_CLS = SecuritySystemSensor
    ITEM_MAPPING = {}

    BYPASS_MODES = {0: "Off", 1: "On"}
    CHIRP_MODES = {0: "Off", 1: "On"}
    TRIGGER_MODES = {
        0: "Off",
        1: "Home",
        2: "Away",
        3: "Home/Away",
    }

    SENSOR_TYPES = {
        1: "Motion Sensor",
        2: "Door/Window Sensor",
    }

    async def initialize_elem(self, device: AferoDevice):
        """Initialize the element.

        :param afero_device: Afero Device that contains the updated states

        :return: Newly initialized resource
        """
        available: bool = False
        numbers: dict[tuple[str, str | None], features.NumbersFeature] | None = {}
        selects: dict[tuple[str, str | None], features.SelectFeature] | None = {}
        sensors: dict[str, AferoSensor] = {}
        binary_sensors: dict[str, AferoBinarySensor] = {}
        device_type: int | None = None
        config_key: str | None = None
        for state in device.states:
            if state.functionClass == "sensor-state":
                data = state.value["security-sensor-state"]
                device_type = data["deviceType"]
                binary_sensors["tampered"] = AferoBinarySensor(
                    id="tampered",
                    owner=device.device_id,
                    instance="tampered",
                    current_value=data["tampered"],
                    _error=1,
                )
                binary_sensors["triggered"] = AferoBinarySensor(
                    id="triggered",
                    owner=device.device_id,
                    instance="triggered",
                    current_value=data["triggered"],
                    _error=1,
                )
                sensors["battery-level"] = AferoSensor(
                    id=state.functionClass,
                    owner=device.device_id,
                    value=data["batteryLevel"],
                    unit="%",
                )
                available = not bool(data["missing"])
            else:
                config_key = list(state.value.keys())[0]
                data = state.value[config_key]
                selects[(state.functionInstance, "chirpMode")] = features.SelectFeature(
                    selected=self.CHIRP_MODES.get(data["chirpMode"]),
                    selects=set(self.CHIRP_MODES.values()),
                    name="Chirp Mode",
                )
                selects[(state.functionInstance, "triggerType")] = (
                    features.SelectFeature(
                        selected=self.TRIGGER_MODES.get(data["triggerType"]),
                        selects=set(self.TRIGGER_MODES.values()),
                        name="Triggers",
                    )
                )
                selects[(state.functionInstance, "bypassType")] = (
                    features.SelectFeature(
                        selected=self.BYPASS_MODES.get(data["bypassType"]),
                        selects=set(self.BYPASS_MODES.values()),
                        name="Can Be Bypassed",
                    )
                )
        self._items[device.id] = SecuritySystemSensor(
            [],
            _id=device.id,
            split_identifier=SENSOR_SPLIT_IDENTIFIER,
            config_key=config_key,
            available=available,
            sensors=sensors,
            binary_sensors=binary_sensors,
            numbers=numbers,
            selects=selects,
            device_information=DeviceInformation(
                device_class=device.device_class,
                default_image=device.default_image,
                default_name=device.default_name,
                manufacturer=device.manufacturerName,
                model=self.SENSOR_TYPES.get(device_type, "Unknown"),
                name=device.friendly_name,
                parent_id=device.device_id,
            ),
        )
        return self._items[device.id]

    async def update_elem(self, afero_device: AferoDevice) -> set[str]:
        """Update the Security System Sensor with the latest API data.

        :param afero_device: Afero Device that contains the updated states

        :return: States that have been modified
        """
        return update_from_states(afero_device.states, self[afero_device.id])

    async def set_state(
        self,
        device_id: str,
        selects: dict[tuple[str, int | None], str] | None = None,
    ) -> None:
        """Set supported feature(s) to fan resource."""
        update_obj = SecuritySystemSensorPut()
        try:
            cur_item: SecuritySystemSensor = self.get_device(device_id)
        except DeviceNotFound:
            self._logger.info("Unable to find device %s", device_id)
            return
        if selects:
            chirp_modes = {y: x for x, y in self.CHIRP_MODES.items()}
            trigger_types = {y: x for x, y in self.TRIGGER_MODES.items()}
            bypass_types = {y: x for x, y in self.BYPASS_MODES.items()}
            # Load the current values as it all needs to be sent
            select_vals = {
                "chirpMode": chirp_modes[
                    cur_item.selects.get(
                        (f"{SENSOR_SPLIT_IDENTIFIER}-{cur_item.instance}", "chirpMode")
                    ).selected
                ],
                "triggerType": trigger_types[
                    cur_item.selects.get(
                        (
                            f"{SENSOR_SPLIT_IDENTIFIER}-{cur_item.instance}",
                            "triggerType",
                        )
                    ).selected
                ],
                "bypassType": bypass_types[
                    cur_item.selects.get(
                        (f"{SENSOR_SPLIT_IDENTIFIER}-{cur_item.instance}", "bypassType")
                    ).selected
                ],
            }
            for select, select_val in selects.items():
                if select[1] == "chirpMode":
                    select_vals["chirpMode"] = chirp_modes[select_val]
                elif select[1] == "triggerType":
                    select_vals["triggerType"] = trigger_types[select_val]
                elif select[1] == "bypassType":
                    select_vals["bypassType"] = bypass_types[select_val]
                else:
                    continue
            update_obj.sensor_config = features.SecuritySensorConfigFeature(
                sensor_id=cur_item.instance, key_name=cur_item.config_key, **select_vals
            )
            await self.update(cur_item.id, obj_in=update_obj)


def update_from_states(
    valid_states: list[AferoState], cur_item: SecuritySystemSensor
) -> set[str]:
    """Update the item from the incoming changes."""
    updated_keys = set()
    for state in valid_states:
        if state.functionClass == "sensor-state":
            data = state.value["security-sensor-state"]
            if data["tampered"] != cur_item.binary_sensors["tampered"].current_value:
                updated_keys.add("tampered")
                cur_item.binary_sensors["tampered"].current_value = data["tampered"]
            if data["triggered"] != cur_item.binary_sensors["triggered"].current_value:
                updated_keys.add("triggered")
                cur_item.binary_sensors["triggered"].current_value = data["triggered"]
            if data["batteryLevel"] != cur_item.sensors["battery-level"].value:
                updated_keys.add("battery-level")
                cur_item.sensors["battery-level"].value = data["batteryLevel"]
            if bool(data["missing"]) == cur_item.available:
                updated_keys.add("available")
                cur_item.available = not bool(data["missing"])
        else:
            top_level_key = list(state.value.keys())[0]
            data = state.value[top_level_key]
            if (
                SecuritySystemSensorController.CHIRP_MODES.get(data["chirpMode"])
                != cur_item.selects[(state.functionInstance, "chirpMode")].selected
            ):
                updated_keys.add("chirpMode")
                cur_item.selects[
                    (state.functionInstance, "chirpMode")
                ].selected = SecuritySystemSensorController.CHIRP_MODES.get(
                    data["chirpMode"]
                )
            if (
                SecuritySystemSensorController.TRIGGER_MODES.get(data["triggerType"])
                != cur_item.selects[(state.functionInstance, "triggerType")].selected
            ):
                updated_keys.add("triggerType")
                cur_item.selects[
                    (state.functionInstance, "triggerType")
                ].selected = SecuritySystemSensorController.TRIGGER_MODES.get(
                    data["triggerType"]
                )
            if (
                SecuritySystemSensorController.BYPASS_MODES.get(data["bypassType"])
                != cur_item.selects[(state.functionInstance, "bypassType")].selected
            ):
                updated_keys.add("bypassType")
                cur_item.selects[
                    (state.functionInstance, "bypassType")
                ].selected = SecuritySystemSensorController.BYPASS_MODES.get(
                    data["bypassType"]
                )
    return updated_keys



"""Representation of an Afero Switch and its corresponding updates."""

from dataclasses import dataclass, field

from aioafero.v1.models import features

from .resource import DeviceInformation, ResourceTypes
from .sensor import AferoBinarySensor, AferoSensor


@dataclass
class Switch:
    """Representation of an Afero Switch."""

    id: str  # ID used when interacting with Afero
    available: bool

    on: dict[str | None, features.OnFeature]
    # Defined at initialization
    split_identifier: str | None = None
    instances: dict = field(default_factory=dict, repr=False, init=False)
    device_information: DeviceInformation = field(default_factory=DeviceInformation)
    sensors: dict[str, AferoSensor] = field(default_factory=dict)
    binary_sensors: dict[str, AferoBinarySensor] = field(default_factory=dict)

    type: ResourceTypes = ResourceTypes.FAN

    def __init__(self, functions: list, **kwargs):  # noqa: D107
        for key, value in kwargs.items():
            if key == "instances":
                continue
            setattr(self, key, value)
        instances = {}
        for function in functions:
            instances[function["functionClass"]] = function.get(
                "functionInstance", None
            )
        self.instances = instances

    def get_instance(self, elem):
        """Lookup the instance associated with the elem."""
        return self.instances.get(elem, None)

    @property
    def instance(self):
        """Instance for the split device."""
        if self.split_identifier:
            return self.id.rsplit(f"-{self.split_identifier}-", 1)[1]
        return None

    @property
    def update_id(self) -> str:
        """ID used when sending updates to Afero API."""
        if self.split_identifier:
            return self.id.rsplit(f"-{self.split_identifier}-", 1)[0]
        return self.id


@dataclass
class SwitchPut:
    """States that can be updated for a Switch."""

    on: features.OnFeature | None = None



"""Representation of an Afero Security System Sensor (derived) and its corresponding updates."""

from dataclasses import dataclass, field

from aioafero.v1.models import features

from .resource import DeviceInformation, ResourceTypes
from .sensor import AferoBinarySensor, AferoSensor


@dataclass
class SecuritySystemSensor:
    """Representation of a security system sensor."""

    _id: str  # ID used when interacting with Afero
    available: bool
    selects: dict[tuple[str, str | None], features.SelectFeature] | None
    config_key: str | None
    # Security System Sensors are always split devices
    split_identifier: str

    # Defined at initialization
    instances: dict = field(default_factory=dict, repr=False, init=False)
    device_information: DeviceInformation = field(default_factory=DeviceInformation)
    sensors: dict[str, AferoSensor] = field(default_factory=dict)
    binary_sensors: dict[str, AferoBinarySensor] = field(default_factory=dict)
    type: ResourceTypes = ResourceTypes.SECURITY_SYSTEM_SENSOR

    def __init__(self, functions: list, **kwargs):  # noqa: D107
        for key, value in kwargs.items():
            if key == "instances":
                continue
            setattr(self, key, value)
        instances = {}
        for function in functions:
            instances[function["functionClass"]] = function.get(
                "functionInstance", None
            )
        self.instances = instances

    @property
    def id(self):
        """ID for the device (split or normal)."""
        return self._id

    @property
    def instance(self):
        """Instance for the split device."""
        return int(self._id.rsplit(f"-{self.split_identifier}-", 1)[1])

    @property
    def update_id(self) -> str:
        """ID used when sending updates to Afero API."""
        return self._id.rsplit(f"-{self.split_identifier}-", 1)[0]


@dataclass
class SecuritySystemSensorPut:
    """States that can be updated for a Security System Sensor."""

    sensor_config: features.SecuritySensorConfigFeature | None = field(
        default_factory=dict, repr=False, init=False
    )



"""Representation of an Afero Security System and its corresponding updates."""

from dataclasses import dataclass, field

from aioafero.v1.models import features

from .resource import DeviceInformation, ResourceTypes
from .sensor import AferoBinarySensor, AferoSensor


@dataclass
class SecuritySystem:
    """Representation of an Afero Security Panel."""

    id: str  # ID used when interacting with Afero
    available: bool

    alarm_state: features.ModeFeature | None
    siren_action: features.SecuritySensorSirenFeature | None
    numbers: dict[tuple[str, str | None], features.NumbersFeature] | None
    selects: dict[tuple[str, str | None], features.SelectFeature] | None

    # Defined at initialization
    instances: dict = field(default_factory=dict, repr=False, init=False)
    device_information: DeviceInformation = field(default_factory=DeviceInformation)
    sensors: dict[str, AferoSensor] = field(default_factory=dict)
    binary_sensors: dict[str, AferoBinarySensor] = field(default_factory=dict)

    type: ResourceTypes = ResourceTypes.SECURITY_SYSTEM

    def __init__(self, functions: list, **kwargs):  # noqa: D107
        for key, value in kwargs.items():
            if key == "instances":
                continue
            setattr(self, key, value)
        instances = {}
        for function in functions:
            instances[function["functionClass"]] = function.get(
                "functionInstance", None
            )
        self.instances = instances

    @property
    def supports_away(self) -> bool:
        """States if the panel supports away mode."""
        return "arm-away" in self.alarm_state.modes

    @property
    def supports_arm_bypass(self) -> bool:
        """States if the panel supports arm-bypass mode."""
        return False

    @property
    def supports_home(self) -> bool:
        """States if the panel supports home mode."""
        return "arm-stay" in self.alarm_state.modes

    @property
    def supports_night(self) -> bool:
        """States if the panel supports night mode."""
        return False

    @property
    def supports_vacation(self) -> bool:
        """States if the panel supports vacation mode."""
        return False

    @property
    def supports_trigger(self) -> bool:
        """States if the panel supports manually triggering."""
        return "alarming-sos" in self.alarm_state.modes

    def get_instance(self, elem):
        """Lookup the instance associated with the elem."""
        return self.instances.get(elem, None)


@dataclass
class SecuritySystemPut:
    """States that can be updated for a Security System."""

    alarm_state: features.ModeFeature | None = None
    siren_action: features.SecuritySensorSirenFeature | None = None
    numbers: dict[tuple[str, str | None], features.NumbersFeature] | None = field(
        default_factory=dict, repr=False, init=False
    )
    selects: dict[tuple[str, str | None], features.SelectFeature] | None = field(
        default_factory=dict, repr=False, init=False
    )



"""Controller holding and managing Afero IoT resources of type `security-system`."""

import copy

from aioafero.device import AferoCapability, AferoDevice, get_function_from_device
from aioafero.errors import DeviceNotFound
from aioafero.util import process_function
from aioafero.v1.models import SecuritySystem, SecuritySystemPut, features
from aioafero.v1.models.resource import DeviceInformation, ResourceTypes

from .base import AferoBinarySensor, AferoSensor, BaseResourcesController, NumbersName
from .event import CallbackResponse

SENSOR_SPLIT_IDENTIFIER = "sensor"


def get_sensor_ids(device) -> set[int]:
    """Determine available sensors from the states."""
    sensor_ids = set()
    for state in device.states:
        if state.functionInstance is None:
            continue
        if state.functionInstance.startswith("sensor-") and state.value is not None:
            sensor_id = int(state.functionInstance.split("-", 1)[1])
            sensor_ids.add(sensor_id)
    return sensor_ids


def generate_sensor_name(afero_device, sensor_id: int) -> str:
    """Generate the name for an instanced element."""
    return f"{afero_device.id}-{SENSOR_SPLIT_IDENTIFIER}-{sensor_id}"


def get_valid_states(afero_states: list, sensor_id: int) -> list:
    """Find states associated with the specific sensor."""
    valid_states: list = []
    for state in afero_states:
        if (
            state.functionClass not in ["sensor-state", "sensor-config"]
            or state.value is None
        ):
            continue
        state_sensor_split = state.functionInstance.rsplit("-", 1)
        state_sensor_id = int(state_sensor_split[1])
        if state_sensor_id != sensor_id:
            continue
        valid_states.append(state)
    return valid_states


def get_valid_functions(afero_functions: list, sensor_id: int) -> list:
    """Find functions associated with the specific sensor."""
    valid_functions: list = []
    for func in afero_functions:
        if func["functionClass"] not in ["sensor-state", "sensor-config"]:
            continue
        sensor_split = func["functionInstance"].rsplit("-", 1)
        state_sensor_id = int(sensor_split[1])
        if state_sensor_id != sensor_id:
            continue
        valid_functions.append(func)
    return valid_functions


def get_sensor_name(afero_capabilities: list[AferoCapability], sensor_id: int) -> str:
    """Get the Afero name for a specific sensor."""
    for capability in afero_capabilities:
        if (
            capability.functionClass != "sensor-state"
            or capability.functionInstance != f"sensor-{sensor_id}"
            or capability.options.get("name") is None
        ):
            continue
        return capability.options["name"]
    return f"Sensor {sensor_id}"


def security_system_callback(afero_device: AferoDevice) -> CallbackResponse:
    """Convert an AferoDevice into multiple devices."""
    multi_devs: list[AferoDevice] = []
    if afero_device.device_class == "security-system":
        for sensor_id in get_sensor_ids(afero_device):
            cloned = copy.deepcopy(afero_device)
            cloned.device_id = generate_sensor_name(afero_device, sensor_id)
            cloned.id = generate_sensor_name(afero_device, sensor_id)
            cloned.split_identifier = SENSOR_SPLIT_IDENTIFIER
            cloned.device_class = ResourceTypes.SECURITY_SYSTEM_SENSOR.value
            cloned.friendly_name = f"{afero_device.friendly_name} - {get_sensor_name(afero_device.capabilities, sensor_id)}"
            cloned.states = get_valid_states(afero_device.states, sensor_id)
            cloned.functions = get_valid_functions(afero_device.functions, sensor_id)
            multi_devs.append(cloned)
    return CallbackResponse(
        split_devices=multi_devs,
        remove_original=False,
    )


class SecuritySystemController(BaseResourcesController[SecuritySystem]):
    """Controller holding and managing Afero IoT resources of type `security-system`."""

    ITEM_TYPE_ID = ResourceTypes.DEVICE
    ITEM_TYPES = [ResourceTypes.SECURITY_SYSTEM]
    ITEM_CLS = SecuritySystem
    ITEM_MAPPING = {"alarm_state": "alarm-state"}
    # Sensors map functionClass -> Unit
    ITEM_SENSORS: dict[str, str] = {
        "alarm-state": None,
        "history-event": None,
        "disarmed-by": None,
    }
    # Binary sensors map key -> alerting value
    ITEM_BINARY_SENSORS: dict[str, str] = {
        "battery-powered": "battery-powered",
    }
    # Elements that map to numbers. func class / func instance to NumbersName
    ITEM_NUMBERS: dict[tuple[str, str | None], NumbersName] = {
        ("arm-exit-delay", "away"): NumbersName(
            unit="seconds", display_name="Arm Exit Delay Away"
        ),
        ("arm-exit-delay", "stay"): NumbersName(
            unit="seconds", display_name="Arm Exit Delay Home"
        ),
        ("temporary-bypass-time", None): NumbersName(
            unit="seconds", display_name="Bypass Time"
        ),
        ("disarm-entry-delay", None): NumbersName(
            unit="seconds", display_name="Disarm Entry Delay"
        ),
        ("siren-alarm-timeout", None): NumbersName(
            unit="seconds", display_name="Siren Timeout"
        ),
    }
    # Elements that map to Select. func class / func instance to name
    ITEM_SELECTS = {
        ("song-id", "alarm"): "Alarm Noise",
        ("volume", "siren"): "Alarm Volume",
        ("bypass-allowed", None): "Enable Temporary Bypass",
        ("song-id", "chime"): "Chime Noise",
        ("volume", "chime"): "Chime Volume",
        ("volume", "entry-delay"): "Entry Delay Volume",
        ("volume", "exit-delay-away"): "Exit Delay Volume Away",
        ("volume", "exit-delay-stay"): "Exit Delay Volume Home",
        # ("dark-mode", None): "KeyPad Dark Mode",
        # ("song-id", "beep"): "KeyPad Noise",
    }
    # Split sensors from the primary payload
    DEVICE_SPLIT_CALLBACKS: dict[str, callable] = {
        ResourceTypes.SECURITY_SYSTEM_SENSOR.value: security_system_callback
    }

    async def disarm(self, device_id: str) -> None:
        """Disarm the system."""
        await self.set_state(device_id, mode="disarmed")

    async def arm_home(self, device_id: str) -> None:
        """Arms the system while someone is home."""
        await self.set_state(device_id, mode="arm-started-stay")

    async def arm_away(self, device_id: str) -> None:
        """Arms the system while no one is home."""
        await self.set_state(device_id, mode="arm-started-away")

    async def alarm_trigger(self, device_id: str) -> None:
        """Manually trigger the alarm."""
        await self.set_state(device_id, mode="alarming-sos")

    async def initialize_elem(self, afero_device: AferoDevice) -> SecuritySystem:
        """Initialize the element.

        :param afero_device: Afero Device that contains the updated states

        :return: Newly initialized resource
        """
        available: bool = False
        alarm_state: features.ModeFeature | None = None
        siren_action: features.SecuritySensorSirenFeature | None = None
        numbers: dict[tuple[str, str | None], features.NumbersFeature] | None = {}
        selects: dict[tuple[str, str | None], features.SelectFeature] | None = {}
        sensors: dict[str, AferoSensor] = {}
        binary_sensors: dict[str, AferoBinarySensor] = {}
        for state in afero_device.states:
            func_def = get_function_from_device(
                afero_device.functions, state.functionClass, state.functionInstance
            )
            if state.functionClass == "available":
                available = state.value
            elif state.functionClass == "alarm-state":
                alarm_state = features.ModeFeature(
                    mode=state.value,
                    modes=set(
                        process_function(afero_device.functions, state.functionClass)
                    ),
                )
            elif sensor := await self.initialize_sensor(state, afero_device.device_id):
                if isinstance(sensor, AferoBinarySensor):
                    binary_sensors[sensor.id] = sensor
                else:
                    sensors[sensor.id] = sensor
            elif number := await self.initialize_number(func_def, state):
                numbers[number[0]] = number[1]
            elif select := await self.initialize_select(afero_device.functions, state):
                selects[select[0]] = select[1]
            elif state.functionClass == "siren-action":
                try:
                    result_code = state.value["security-siren-action"]["resultCode"]
                    command = state.value["security-siren-action"]["command"]
                except TypeError:
                    result_code = None
                    command = None
                siren_action = features.SecuritySensorSirenFeature(
                    result_code=result_code,
                    command=command,
                )

        self._items[afero_device.id] = SecuritySystem(
            afero_device.functions,
            id=afero_device.id,
            available=available,
            sensors=sensors,
            binary_sensors=binary_sensors,
            numbers=numbers,
            selects=selects,
            device_information=DeviceInformation(
                device_class=afero_device.device_class,
                default_image=afero_device.default_image,
                default_name=afero_device.default_name,
                manufacturer=afero_device.manufacturerName,
                model=afero_device.model,
                name=afero_device.friendly_name,
                parent_id=afero_device.device_id,
                children=afero_device.children,
                functions=afero_device.functions,
            ),
            alarm_state=alarm_state,
            siren_action=siren_action,
        )
        return self._items[afero_device.id]

    async def update_elem(self, afero_device: AferoDevice) -> set:
        """Update the Security System with the latest API data.

        :param afero_device: Afero Device that contains the updated states

        :return: States that have been modified
        """
        cur_item = self.get_device(afero_device.id)
        updated_keys = set()
        for state in afero_device.states:
            if state.functionClass == "available":
                if cur_item.available != state.value:
                    updated_keys.add("available")
                cur_item.available = state.value
            elif state.functionClass == "alarm-state":
                if cur_item.alarm_state.mode != state.value:
                    updated_keys.add(state.functionClass)
                cur_item.alarm_state.mode = state.value
            elif (
                (update_key := await self.update_sensor(state, cur_item))
                or (update_key := await self.update_number(state, cur_item))
                or (update_key := await self.update_select(state, cur_item))
            ):
                updated_keys.add(update_key)
            elif state.functionClass == "siren-action":
                try:
                    result_code = state.value["security-siren-action"]["resultCode"]
                    command = state.value["security-siren-action"]["command"]
                except TypeError:
                    result_code = None
                    command = None
                if (
                    result_code != cur_item.siren_action.result_code
                    or command != cur_item.siren_action.command
                ):
                    cur_item.siren_action.result_code = result_code
                    cur_item.siren_action.command = command
                    updated_keys.add("siren-action")

        return updated_keys

    async def set_state(
        self,
        device_id: str,
        mode: str | None = None,
        numbers: dict[tuple[str, str | None], float] | None = None,
        selects: dict[tuple[str, str | None], str] | None = None,
    ) -> None:
        """Set supported feature(s) to Security System resource."""
        update_obj = SecuritySystemPut()
        try:
            cur_item = self.get_device(device_id)
        except DeviceNotFound:
            self._logger.info("Unable to find device %s", device_id)
            return
        if mode is not None:
            update_obj.alarm_state = features.ModeFeature(
                mode=mode,
                modes=cur_item.alarm_state.modes,
            )
            if "-started-" in mode:
                update_obj.siren_action = features.SecuritySensorSirenFeature(
                    result_code=0,
                    command=4,
                )
            elif mode == "alarming-sos":
                update_obj.siren_action = features.SecuritySensorSirenFeature(
                    result_code=0,
                    command=5,
                )
            else:
                update_obj.siren_action = features.SecuritySensorSirenFeature(
                    result_code=None,
                    command=None,
                )
        if numbers:
            for key, val in numbers.items():
                if key not in cur_item.numbers:
                    continue
                update_obj.numbers[key] = features.NumbersFeature(
                    value=val,
                    min=cur_item.numbers[key].min,
                    max=cur_item.numbers[key].max,
                    step=cur_item.numbers[key].step,
                    name=cur_item.numbers[key].name,
                    unit=cur_item.numbers[key].unit,
                )
        if selects:
            for key, val in selects.items():
                if key not in cur_item.selects:
                    continue
                update_obj.selects[key] = features.SelectFeature(
                    selected=val,
                    selects=cur_item.selects[key].selects,
                    name=cur_item.selects[key].name,
                )
        await self.update(device_id, obj_in=update_obj)



"""Representation of an Afero Light and its corresponding updates."""

from dataclasses import dataclass, field
import re

from aioafero.v1.models import features

from .resource import DeviceInformation, ResourceTypes
from .sensor import AferoBinarySensor, AferoSensor

rgbw_name_search = re.compile(r"RGB\w*W")


@dataclass
class Light:
    """Representation of an Afero Light."""

    id: str  # ID used when interacting with Afero
    available: bool

    on: features.OnFeature | None
    color: features.ColorFeature | None
    color_mode: features.ColorModeFeature | None
    color_modes: list[str] | None
    color_temperature: features.ColorTemperatureFeature | None
    dimming: features.DimmingFeature | None
    effect: features.EffectFeature | None
    supports_white: bool = False

    # Defined at initialization
    split_identifier: str | None = None
    instances: dict = field(default_factory=dict, repr=False, init=False)
    device_information: DeviceInformation = field(default_factory=DeviceInformation)
    sensors: dict[str, AferoSensor] = field(default_factory=dict)
    binary_sensors: dict[str, AferoBinarySensor] = field(default_factory=dict)

    type: ResourceTypes = ResourceTypes.LIGHT

    def __init__(self, functions: list, **kwargs):  # noqa: D107
        for key, value in kwargs.items():
            if key == "instances":
                continue
            setattr(self, key, value)
        instances = {}
        for function in functions:
            instances[function["functionClass"]] = function.get(
                "functionInstance", None
            )
        self.instances = instances
        self.supports_white = (
            rgbw_name_search.search(self.device_information.model) is not None
        )

    def get_instance(self, elem):
        """Lookup the instance associated with the elem."""
        return self.instances.get(elem, None)

    @property
    def instance(self):
        """Instance for the split device."""
        if self.split_identifier:
            return self.id.rsplit(f"-{self.split_identifier}-", 1)[1]
        return None

    @property
    def update_id(self) -> str:
        """ID used when sending updates to Afero API."""
        if self.split_identifier:
            return self.id.rsplit(f"-{self.split_identifier}-", 1)[0]
        return self.id

    @property
    def supports_color(self) -> bool:
        """Return if this light supports color control."""
        return self.color is not None

    @property
    def supports_color_temperature(self) -> bool:
        """Return if this light supports color_temperature control."""
        return self.color_temperature is not None

    @property
    def supports_color_white(self) -> bool:
        """Return if this light supports setting white."""
        return self.color_modes is not None and "white" in self.color_modes

    @property
    def supports_dimming(self) -> bool:
        """Return if this light supports brightness control."""
        return self.dimming is not None

    @property
    def supports_effects(self) -> bool:
        """Return if this light supports brightness control."""
        return self.effect is not None

    @property
    def supports_on(self):
        """If the light can be toggled on or off."""
        return self.on is not None

    @property
    def is_on(self) -> bool:
        """Return bool if light is currently powered on."""
        if self.on is not None:
            return self.on.on
        return False

    @property
    def brightness(self) -> float:
        """Return current brightness of light."""
        if self.dimming is not None:
            return self.dimming.brightness
        return 100.0 if self.is_on else 0.0


@dataclass
class LightPut[AferoResource]:
    """States that can be updated for a light."""

    on: features.OnFeature | None = None
    color: features.ColorFeature | None = None
    color_mode: features.ColorModeFeature | None = None
    color_temperature: features.ColorTemperatureFeature | None = None
    dimming: features.DimmingFeature | None = None
    effect: features.EffectFeature | None = None



import pytest

from aioafero.v1.models import features
from aioafero.v1.models.switch import Switch


@pytest.fixture
def populated_entity():
    return Switch(
        [
            {
                "functionClass": "preset",
                "functionInstance": "preset-1",
                "value": "on",
                "lastUpdateTime": 0,
            }
        ],
        id="entity-1",
        available=True,
        on={None: features.OnFeature(on=True)},
        instances="i dont execute",
    )


@pytest.fixture
def empty_entity():
    return Switch(
        [
            {
                "functionClass": "preset",
                "functionInstance": "preset-1",
                "value": "on",
                "lastUpdateTime": 0,
            }
        ],
        id="entity-1",
        available=True,
        on=None,
        instances="i dont execute",
    )


def test_init(populated_entity):
    assert populated_entity.id == "entity-1"
    assert populated_entity.available is True
    assert populated_entity.instances == {"preset": "preset-1"}
    assert populated_entity.on[None].on is True
    assert populated_entity.update_id == "entity-1"
    assert populated_entity.instance is None
    populated_entity.id = "entity-beans-1"
    populated_entity.split_identifier = "beans"
    assert populated_entity.update_id == "entity"
    assert populated_entity.instance == "1"


def test_init_empty(empty_entity):
    assert not empty_entity.on


def test_get_instance(populated_entity):
    assert populated_entity.get_instance("preset") == "preset-1"
